@@ -1,5 +1,7 @@
 """The graph module."""
 from squidpy.gr._build import spatial_neighbors
 from squidpy.gr._nhood import nhood_enrichment, centrality_scores, interaction_matrix
+from squidpy.gr._sepal import sepal
 from squidpy.gr._ligrec import ligrec
-from squidpy.gr._ppatterns import moran, ripley_k, co_occurrence
+from squidpy.gr._ripley import ripley
+from squidpy.gr._ppatterns import co_occurrence, spatial_autocorr
