@@ -49,7 +49,7 @@ class CustomDotplot(sc.pl.DotPlot):
         # no point in showing dot of size 0
         size_range = size_range[1:]
 
-        size = size_range ** self.size_exponent
+        size = size_range**self.size_exponent
         mult = (self.largest_dot - self.smallest_dot) + self.smallest_dot
         size = size * mult
 
@@ -183,7 +183,7 @@ def ligrec(
         Whether to show the cluster combinations as rows and the interacting pairs as columns.
     title
         Title of the plot.
-    %(plotting)s
+    %(plotting_save)s
     kwargs
         Keyword arguments for :meth:`scanpy.pl.DotPlot.style` or :meth:`scanpy.pl.DotPlot.legend`.
 
@@ -268,8 +268,8 @@ def ligrec(
     if title is None:
         title = "Receptor-ligand test"
 
-    source_groups, _ = _unique_order_preserving(source_groups)  # type: ignore[no-redef,assignment]
-    target_groups, _ = _unique_order_preserving(target_groups)  # type: ignore[no-redef,assignment]
+    source_groups, _ = _unique_order_preserving(source_groups)  # type: ignore[assignment]
+    target_groups, _ = _unique_order_preserving(target_groups)  # type: ignore[assignment]
 
     pvals: pd.DataFrame = adata["pvalues"].loc[:, (source_groups, target_groups)]
     means: pd.DataFrame = adata["means"].loc[:, (source_groups, target_groups)]
@@ -311,7 +311,7 @@ def ligrec(
     var = pd.DataFrame(pvals.columns)
     var = var.set_index(var.columns[0])
 
-    adata = AnnData(pvals.values, obs={"groups": pd.Categorical(pvals.index)}, var=var)
+    adata = AnnData(pvals.values, obs={"groups": pd.Categorical(pvals.index)}, var=var, dtype=pvals.values.dtype)
     adata.obs_names = pvals.index
     minn = np.nanmin(adata.X)
     delta = np.nanmax(adata.X) - minn
